#!/bin/sh

KIT_ROOT=$(pwd)

echo "installing kit from $KIT_ROOT"

echo 'installing package prereqs'
sudo apt-get install build-essential xterm tmux vim perl ack-grep \
  git git-gui htop zsh suckless-tools figlet xmonad trayer xmobar \
  volumeicon-alsa curl jq byzanz festival clipit redshift-gtk ranger \
  rofi xsettingsd wmctrl lynx perl-doc dict moreutils myrepos etckeeper \
  bzr cvs darcs mercurial subversion subversion-tools dunst i3lock \
  fzf xclip xautolock pngphoon tree scrot pandoc dnsutils \
  pasystray pavucontrol

echo 'installing perl dependencies'
sudo cpan -i Module::Build App::WRT

echo 'vim setup'
  mkdir -p ~/.vim/bundle
  if [ ! -f ~/.vimrc ]; then
    ln -s "$KIT_ROOT/home/.vimrc" "$HOME/.vimrc"
  fi
  if [ ! -f ~/.gvimrc ]; then
    ln -s "$KIT_ROOT/home/.gvimrc" "$HOME/.gvimrc"
  fi
  if [ ! -d ~/.vim/bundle/Vundle.vim ]; then
    git clone https://github.com/gmarik/vundle.git ~/.vim/bundle/Vundle.vim
  fi
  vim +PluginInstall

echo 'xmonad setup'
echo "(don't forget to point /usr/share/xsessions/xmonad.desktop at xmonad.start)"
  if [ ! -d ~/.xmonad ]; then
    ln -s "$KIT_ROOT/home/.xmonad" "$HOME/.xmonad"
  fi
  if [ ! -f ~/.xmobarrc ]; then
    ln -s "$KIT_ROOT/home/.xmobarrc" "$HOME/.xmobarrc"
  fi
  if [ ! -f ~/.xsettingsd ]; then
    ln -s "$KIT_ROOT/home/.xsettingsd" "$HOME/.xsettingsd"
  fi

# shellcheck disable=SC2088
echo '~/bin'
  if [ ! -d ~/bin ]; then
    ln -s "$KIT_ROOT/home/bin" "$HOME/bin"
  fi

# shellcheck disable=SC2088
echo '~/fragments'
  if [ ! -d ~/fragments ]; then
    ln -s "$KIT_ROOT/home/fragments" "$HOME/fragments"
  fi

# shellcheck disable=SC2088
echo '~/cheatsheets'
  if [ ! -d ~/cheatsheets ]; then
    ln -s "$KIT_ROOT/home/cheatsheets" "$HOME/cheatsheets"
  fi

echo 'shell setup'
  if [ ! -f ~/.zshrc ]; then
    ln -s "$KIT_ROOT/home/.zshrc" "$HOME/.zshrc"
  fi
  if [ ! -f ~/.bashrc ]; then
    ln -s "$KIT_ROOT/home/.bashrc" "$HOME/.bashrc"
  fi
  if [ ! -f ~/.sh_common ]; then
    ln -s "$KIT_ROOT/home/.sh_common" "$HOME/.sh_common"
  fi

echo 'tmux / screen setup'
  if [ ! -f ~/.tmux.conf ]; then
    ln -s "$KIT_ROOT/home/.tmux.conf" "$HOME/.tmux.conf"
  fi
  if [ ! -f ~/.screenrc ]; then
    ln -s "$KIT_ROOT/home/.screenrc" "$HOME/.screenrc"
  fi

echo 'git setup'
  if [ ! -f ~/.gitconfig ]; then
    ln -s "$KIT_ROOT/home/.gitconfig" "$HOME/.gitconfig"
  fi

echo 'lynx setup'
  if [ ! -f ~/.lynxrc ]; then
    ln -s "$KIT_ROOT/home/.lynxrc" "$HOME/.lynxrc"
  fi

echo 'keyboard setup (Ctrl and CapsLock swap)'
  sudo cp etc/default/keyboard /etc/default/keyboard

echo 'Remember to:'
echo '  - install commandlog from https://code.p1k3.com/gitea/brennen/commandlog'
echo '  - chsh'
